#include <iostream>
#include <sstream>

#include <TROOT.h>
#include <TApplication.h>
#include <TNtupleD.h>
#include <TH1D.h>
#include <TFile.h>

#include "MediumMagboltz.hh"
#include "SolidBox.hh"
#include "GeometrySimple.hh"
#include "ComponentConstant.hh"
#include "Sensor.hh"
#include "TrackHeed.hh"
#include "Plotting.hh"
#include "Random.hh"

using namespace Garfield;

int main(int argc, char * argv[]) {

  randomEngine.Seed(123456);
  TApplication app("app", &argc, argv);
  plottingEngine.SetDefaultStyle();

  // Make a medium
  MediumMagboltz* gas = new MediumMagboltz();
  gas->SetComposition("ar", 95., "c4h10", 2., "cf4", 3.);
  gas->SetTemperature(293.15);
  gas->SetPressure(760.);

  // Detector geometry
  // Gap [cm]
  const double width = 1.;
  //const double width = 0.536; // [cm] LP1 Asian GEM module
  SolidBox* box = new SolidBox(width / 2., 0., 0., width / 2., 10., 10.);
  GeometrySimple* geo = new GeometrySimple();
  geo->AddSolid(box, gas);

  // Make a component
  ComponentConstant* comp = new ComponentConstant();
  comp->SetGeometry(geo);
  comp->SetElectricField(100., 0., 0.);

  // Make a sensor
  Sensor* sensor = new Sensor();
  sensor->AddComponent(comp);

  // Track class
  TrackHeed* track = new TrackHeed();
  track->SetSensor(sensor);
#if 1
  std::string pid("electron");
  track->SetParticle(pid.data());
  track->SetMomentum(5.e9);
#else
  std::string pid("pi");
  track->SetParticle(pid.data());
  track->SetMomentum(0.56e9);
#endif

  // Output file
  std::stringstream foutstr;
  foutstr << "edep" << "." << pid.data() 
	            << "." << track->GetMomentum() << ".root" << std::ends;
  TFile *hfp = new TFile(foutstr.str().data(),"RECREATE","Marilin");

  // N-tuples
  static TNtupleD *hHits    = 0;
  if (!hHits) {
    std::stringstream list;
    list << "nsum:esum:np" << std::ends;
    hHits = new TNtupleD("hHits","",list.str().data());
  }
  auto hNcp = new TNtupleD("hNc","","nc:ec");

  static const Int_t kNdata = 10000;
  Double_t data[kNdata];

  const int nEvents = 1000000;
  //const int nEvents = 100000;
  track->EnableDebugging();
  for (int i = 0; i < nEvents; ++i) {
    if (i == 1) track->DisableDebugging();
    if (i % 1000 == 0) std::cout << i << "/" << nEvents << "\n";
    // Initial position and direction 
    double x0 = 0., y0 = 0., z0 = 0., t0 = 0.;
    double dx0 = 1., dy0 = 0., dz0 = 0.; 
    track->NewTrack(x0, y0, z0, t0, dx0, dy0, dz0);
    // Cluster coordinates
    double xc = 0., yc = 0., zc = 0., tc = 0.;
    // Number of electrons produced in a collision
    int nc = 0;
    // Energy loss in a collision
    double ec = 0.;
    // Dummy variable (not used at present)
    double extra = 0.;
    // Total energy loss along the track
    double esum = 0.;
    // Total number of electrons produced along the track
    int nsum = 0;
    int np   = 0;
    int k2sum = 0;
    // Loop over the clusters.
    while (track->GetCluster(xc, yc, zc, tc, nc, ec, extra)) {
      esum += ec;
      nsum += nc;
      k2sum += nc*nc;
      np++;
      hNcp->Fill(nc,ec);
    }
    data[0] = nsum;
    data[1] = esum*1.e-3; // [keV]
    data[2] = np;
    data[3] = k2sum;
    hHits->Fill(data);
  }

  hfp->Write();

  //app.Run(kTRUE); 
}
